@@ -1,9 +1,9 @@
 #!/bin/sh
 #
-#    Project: Fabio Input/Output
-#             https://github.com/silx-kit/fabio
+#    Project: PyFAI Input/Output
+#             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2023 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -29,20 +29,20 @@
 
 project=pyFAI
 source_project=pyfai
-version=$(python -c"import version; print(version.version)")
-strictversion=$(python -c"import version; print(version.strictversion)")
-debianversion=$(python -c"import version; print(version.debianversion)")
+version=$(python3 -c"import version; print(version.version)")
+strictversion=$(python3 -c"import version; print(version.strictversion)")
+debianversion=$(python3 -c"import version; print(version.debianversion)")
 
 deb_name=$(echo "$source_project" | tr '[:upper:]' '[:lower:]')
 
 # target system
 if [ -f /etc/debian_version ]
-then 
+then
     debian_version=$(cat /etc/debian_version | cut -d. -f1 | grep -o '[0-9]*')
     if [ -z $debian_version ]
     then
     #we are probably on a ubuntu platform
-        debian_version=$(cat /etc/debian_version | cut -d/ -f1)
+        debian_version=$(< /etc/debian_version  cut -d/ -f1)
         case $debian_version in
             squeeze)
                 debian_version=6
@@ -56,6 +56,15 @@ then
             stretch)
                 debian_version=9
                 ;;
+            buster)
+                debian_version=10
+                ;;
+            bullseye)
+                debian_version=11
+                ;;
+            bookworm)
+                debian_version=12
+                ;;
         esac
     fi
 
@@ -72,6 +81,7 @@ build_directory=${project_directory}/build/${target_system}
 if [ -d /usr/lib/ccache ];
 then
    export PATH=/usr/lib/ccache:$PATH
+   export CCACHE_DIR=${HOME}/.ccache
 fi
 
 usage="usage: $(basename "$0") [options]
@@ -82,16 +92,19 @@ If the build succeed the directory dist/debian${debian_version} will
 contains the packages.
 
 optional arguments:
-    --help     show this help text
-    --install  install the packages generated at the end of
-               the process using 'sudo dpkg'
-    --debian7  Simulate a debian7 system (fail-safe)
-    --debian8  Simulate a debian 8 Jessie system
-    --debian9  Simulate a debian 9 Stretch system
+    --help          Show this help text
+    --install       Install the packages generated at the end of
+                    the process using 'sudo dpkg'
+    --stdeb-py3     Build using stdeb for python3
+    --debian9       Simulate a debian 9 Stretch system
+    --debian10      Simulate a debian 10 Buster system
+    --debian11      Simulate a debian 11 Bullseye system
+    --debian12      Simulate a debian 12 Bookworm system
 "
 
 install=0
-use_python3=0 #used only for stdeb
+use_stdeb=0
+stdeb_all_python=0
 
 while :
 do
@@ -104,22 +117,14 @@ do
           install=1
           shift
           ;;
-      --python3)
-          use_python3=1
+      --stdeb-py2)
+          use_stdeb=1
+          stdeb_all_python=0
           shift
           ;;
-      --debian7)
-          debian_version=7
-          target_system=debian${debian_version}
-          dist_directory=${project_directory}/dist/${target_system}
-          build_directory=${project_directory}/build/${target_system}
-          shift
-          ;;
-      --debian8)
-          debian_version=8
-          target_system=debian${debian_version}
-          dist_directory=${project_directory}/dist/${target_system}
-          build_directory=${project_directory}/build/${target_system}
+      --stdeb-py2.py3)
+          use_stdeb=1
+          stdeb_all_python=1
           shift
           ;;
       --debian9)
@@ -129,6 +134,27 @@ do
           build_directory=${project_directory}/build/${target_system}
           shift
           ;;
+      --debian10)
+          debian_version=10
+          target_system=debian${debian_version}
+          dist_directory=${project_directory}/dist/${target_system}
+          build_directory=${project_directory}/build/${target_system}
+          shift
+          ;;
+      --debian11)
+          debian_version=11
+          target_system=debian${debian_version}
+          dist_directory=${project_directory}/dist/${target_system}
+          build_directory=${project_directory}/build/${target_system}
+          shift
+          ;;
+      --debian12)
+          debian_version=12
+          target_system=debian${debian_version}
+          dist_directory=${project_directory}/dist/${target_system}
+          build_directory=${project_directory}/build/${target_system}
+          shift
+          ;;
       -*)
           echo "Error: Unknown option: $1" >&2
           echo "$usage"
@@ -149,11 +175,19 @@ clean_up()
     mkdir -p ${build_directory}
 }
 
-build_deb_8_plus () {
-    echo "Build for debian 8 or newer using actual packaging" 
+build_deb() {
+    echo "Build for debian 9 or newer using actual packaging"
     tarname=${project}_${debianversion}.orig.tar.gz
     clean_up
-    python setup.py debian_src
+    if [ $debian_version -le 11 ]
+    then
+       python3 setup.py debian_src
+       directory=${project}-${strictversion}
+    else
+       python3 -m build -s
+       ln -s ${source_project}-${strictversion}.tar.gz dist/${tarname}
+       directory=${source_project}-${strictversion}
+    fi
     cp -f dist/${tarname} ${build_directory}
     if [ -f dist/${project}-testimages.tar.gz ]
     then
@@ -163,7 +197,7 @@ build_deb_8_plus () {
     cd ${build_directory}
     tar -xzf ${tarname}
 
-    directory=${project}-${strictversion}
+
     newname=${deb_name}_${debianversion}.orig.tar.gz
 
     #echo tarname $tarname newname $newname
@@ -206,8 +240,24 @@ build_deb_8_plus () {
       #export DEB_BUILD_OPTIONS=nocheck
     fi
 
-    dch -v ${debianversion}-1 "upstream development build of ${project} ${version}"
-    dch --bpo "${project} snapshot ${version} built for ${target_system}"
+    case $debian_version in
+        9)
+            debian_name=stretch
+            ;;
+        10)
+            debian_name=buster
+            ;;
+        11)
+            debian_name=bullseye
+            ;;
+        12)
+            debian_name=bookworm
+            ;;
+    esac
+
+    dch --force-distribution -v ${debianversion}-1 "upstream development build of ${project} ${version}"
+    dch --force-distribution -D ${debian_name}-backports -l~bpo${debian_version}+ "${project} snapshot ${version} built for ${target_system}"
+    #dch --bpo "${project} snapshot ${version} built for ${target_system}"
     dpkg-buildpackage -r
     rc=$?
 
@@ -227,26 +277,27 @@ build_deb_8_plus () {
     fi
 }
 
-build_deb_7_minus () {
-    echo "Build for debian 7 or older using stdeb"
+
+build_stdeb () {
+    echo "Build for debian using stdeb"
     tarname=${project}-${strictversion}.tar.gz
     clean_up
 
-    python setup.py sdist
+    python3 setup.py sdist
     cp -f dist/${tarname} ${build_directory}
     cd ${build_directory}
     tar -xzf ${tarname}
     cd ${project}-${strictversion}
 
-    if [ $use_python3 = 1 ]
+    if [ $stdeb_all_python -eq 1  ]
     then
       echo Using Python 2+3
       python3 setup.py --command-packages=stdeb.command sdist_dsc --with-python2=True --with-python3=True --no-python3-scripts=True build --no-cython bdist_deb
       rc=$?
     else
-      echo Using Python 2
+      echo Using Python 3
       # bdist_deb feed /usr/bin using setup.py entry-points
-      python setup.py --command-packages=stdeb.command build --no-cython bdist_deb
+      python3 setup.py --command-packages=stdeb.command build --no-cython bdist_deb
       rc=$?
     fi
 
@@ -259,15 +310,16 @@ build_deb_7_minus () {
     cd ../../..
 }
 
-if [ $debian_version -ge 8 ]
-then 
-    build_deb_8_plus
+
+if [ $use_stdeb -eq 1 ]; then
+    build_stdeb
 else
-    build_deb_7_minus
+    build_deb
 fi
 
+
 if [ $install -eq 1 ]; then
-  sudo -v su -c  "dpkg -i ${dist_directory}/*.deb"
+  sudo su -c  "dpkg -i ${dist_directory}/*.deb"
 fi
 
 exit "$rc"
